;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* 
;* Number		: AVR410
;* File Name            :"rc5.asm"
;* Title                :RC5 IR Remote Control Decoder
;* Date                 :97.08.15
;* Version              :1.0
;* Support telephone    :+47 72 88 43 88 (ATMEL Norway)
;* Support fax          :+47 72 88 43 99 (ATMEL Norway)
;* Target MCU           :AT90S1200
;*
;* DESCRIPTION
;* This Application note describes how to decode the frequently used
;* RC5 IR remote control protocol.
;*
;* The timing is adapted for 4 MHz crystal
;*
;***************************************************************************




.equ	INPUT		=0	;PD0
.equ	SYS_ADDR	=0	;The system address


.def	S	=R5
.def	inttemp	=R6
.def	ref1	=R7
.def	ref2	=R8

.def	temp	=R31

.def	timerL	=R27
.def	timerH	=R28

.def	system	=R30
.def	command	=R29

.def	bitcnt	=R26




.def    error = r24












;********************************************************************
;* Example program
;*
;* Initializes timer, ports and interrupts.
;*
;* Calls "detect" in an endless loop and puts the result out on
;* port B.
;*
;* Number of words:	16
;* Low registers used:	0
;* High registers used: 3
;* Pointers used:	0
;********************************************************************

IR_Start:		
		ldi	temp,1		;Timer/Counter 0 clocked at CK
		out	TCCR0,temp

		ldi	temp,1<<TOIE0	;Enable Timer0 overflow interrupt
		out	TIMSK,temp

		sei			;Enable gobal iterrupt
	ldi error,0;
		rjmp    detect;


ir_main:
		
		ret;



;********************************************************************
;* "detect" - RC5 decode routine
;*
;* This subroutine decodes the RC5 bit stream applied on PORTD
;* pin "INPUT". 
;*
;* If successe: The command and system address are
;*		returned in "command" and "system".
;*		Bit 6 of "command" holds the toggle bit.
;*
;* If failed: 	$FF in both "system" and "command"
;*
;* Crystal frequency is 4MHz
;*
;* Number of words:	72
;* Low registers used:	 3
;* High registers used:  6
;* Pointers used:	 0
;********************************************************************
detect:		clr	inttemp
		clr	timerH
	

detect1:	clr	timerL

detect2:	cpi	timerH,8	;If line not idle within 131ms
		brlo	dl1
		rjmp	fault		;  then exit

dl1:		cpi	timerL,55	;If line low for 3.5ms
		brge	start1		;   then wait for start bit

		sbis	PIND,INPUT	;If line is
		rjmp	detect1		;   low  - jump to detect1
		rjmp	detect2		;   high - jump to detect2


start1:		cpi	timerH,8	;If no start bit detected
		brge	fault		;within 130ms then exit
   	
  		sbic	PIND,INPUT	;Wait for start bit
		rjmp	start1


	ldi error,1;			error=1 if startbit found
		clr	timerL		;Measure length of start bit
		
start2:		cpi	timerL,30	;If startbit longer than 1.1ms,
		brge	fault		;   exit

		sbis	PIND,INPUT
		rjmp	start2
					;Positive edge of 1st start bit
	
	ldi error,2;		      error=2 if startbit short enough
		mov	temp,timerL	;timer is 1/2 bit time
		clr	timerL

		mov	ref1,temp
		lsr	ref1
		mov	ref2,ref1
		add	ref1,temp	;ref1 = 3/4 bit time
		lsl	temp
		add	ref2,temp	;ref2 = 5/4 bit time


;start3:		cp	timerL,ref1	;If high periode St2 > 3/4 bit time
start3:		cp	timerL,ref2	;If high periode St2 > 3/4 bit time
		brge	fault		;   exit


		sbic	PIND,INPUT	;Wait for falling edge start bit 2
		rjmp	start3

		clr	timerL
		ldi	bitcnt,12	;Receive 12 bits
		clr	command
		clr	system

	ldi error,3;		      error=2 if start sampling
sample:		cp	timerL,ref1	;Sample INPUT at 1/4 bit time
		brlo	sample

		sbic	PIND,INPUT
		rjmp	bit_is_a_1	;Jump if line high


bit_is_a_0:	clc			;Store a '0'
		rol	command
		rol	system

					;Synchronize timing
bit_is_a_0a:	cp	timerL,ref2	;If no edge within 3/4 bit time
		brge	fault		;   exit
		sbis	PIND,INPUT	;Wait for rising edge
		rjmp	bit_is_a_0a	;in the middle of the bit

		clr	timerL
		rjmp	nextbit

bit_is_a_1:	sec			;Store a '1'
		rol	command
		rol	system
					;Synchronize timing
bit_is_a_1a:	cp	timerL,ref2	;If no edge within 3/4 bit time
		brge	fault		;   exit
		sbic	PIND,INPUT	;Wait for falling edge
		rjmp	bit_is_a_1a	;in the middle of the bit

		clr	timerL

nextbit:	dec	bitcnt		;If bitcnt > 0
		brne	sample		;   get next bit


;All bits sucessfully received!
		mov	temp,command	;Place system bits in "system"
		rol	temp
		rol	system
		rol	temp
		rol	system

		bst	system,5	;Move toggle bit
		bld	command,6	;to "command"

					;Clear remaining bits
		andi	command,0b01111111
		andi	system,0x1F

		rjmp ir_main;

fault:		
		ser	command		;Both "command" and "system"
		ser	system		;0xFF indicates failure
		rjmp ir_main;
