;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* 
;* Number		: AVR410
;* File Name            :"rc5.asm"
;* Title                :RC5 IR Remote Control Decoder
;* Date                 :97.08.15
;* Version              :1.0
;* Support telephone    :+47 72 88 43 88 (ATMEL Norway)
;* Support fax          :+47 72 88 43 99 (ATMEL Norway)
;* Target MCU           :AT90S1200
;*
;* DESCRIPTION
;* This Application note describes how to decode the frequently used
;* RC5 IR remote control protocol.
;*
;* The timing is adapted for 4 MHz crystal
;*
;***************************************************************************




.equ	INPUT		=0	;PD0
.equ	SYS_ADDR	=0	;The system address


.equ    tbit4		=26	;TimerL ticks for bit*1/4
.equ    tbit2		=53	;TimerL ticks for bit*1/2
.equ    tbit3		=79	;TimerL ticks for bit*3/4
.equ    tbit5		=131	;TimerL ticks for bit*5/4
;	@38 KHz -> Bittime / 2 = 842s =>Bittime = 105,25 ticks
;	Timer tick =  16 s @ 4 MHz
.equ	t133ms		=33	;TimerH ticks for 133ms
;	TimerH tick =  4096 s @ 4 MHz



.def	S	=R5
.def	inttemp	=R6
.def	ref1	=R7
.def	ref2	=R8

.def	TimerH  =R27		;updated every 4096 s
.def	temp	=R31
.def	system	=R30
.def	command	=R29

.def	bitcnt	=R26




.def    error = r24



IR_Init:	cli			;
		ldi	temp,3		;Timer/Counter 0 clocked at CK/64
		out	TCCR0,temp

		;ldi	temp,2		;Enable Timer0 overflow interrupt
		;out	TIMSK,temp

		sei			;Enable gobal iterrupt
		ret;



;**************************************************************************
IR_REC:
		clr temp;
		out TCNT0,temp;
		clr timerH;
		

	; *** test if idle for minimum 3.5ms					
IR_det_idle: 	
		in temp,TCNT0
		cpi temp,218;		
		brge IR_det_sb;
				
		sbis PIND,Input;	LO at Input?
		rjmp fault;		Yes -> failure !
		
		rjmp IR_det_idle;
		

	; *** wait for startbit (maximum 130ms)


		clr temp;
		out TCNT0,temp;
		clr TimerH;

IR_det_sb:	in temp,TCNT0
		cpi temp,$FE;		
		brlo IR_noovf;
		inc TimerH;
		
		IR_noovf:
		cpi TimerH,t133ms;		
		brge fault;
		
		mov temp,sw_status;
		cpi temp,0;
		brne fault;
				
		sbis PIND,Input;	LO at Input?
		rjmp IR_sb1;		NO -> Startbit 1 !
		
		rjmp IR_det_sb;
				
		
	; *** startbit 1 starts now
		
IR_SB1:		

	ldi command,80;
	ret;

		clr temp;
		out TCNT0,temp;
IR_SB1_1:	in temp,TCNT0
		cpi temp,218;		pulse longer than 3.5ms?
		brge fault;		-> Yes -> failure !		
		
		sbis PIND,Input;	wait until HI
		rjmp IR_SB1_1;
		
	;*** startbit 1 first half is gone now

	ldi command,80;
	ret;







		clr temp;
		out TCNT0,temp;
					
		
IR_SB1_2:	in temp,TCNT0
		cpi temp,218;		pulse longer than 3.5ms?
		brge fault;
		
		sbic PIND,Input;	Input Lo ? -> Yes -> startbit 2!
		rjmp IR_SB1_2;

	;*** startbit 2 starts now
		;*** startbit 2 starts now
		





IR_SB2:		in temp,TCNT0
		cpi temp,218;		pulse longer than 3.5ms?
		brge fault;
				
		sbis PIND,Input;	wait for Startbit 2 low
		rjmp IR_SB2;
			
	;*** get system adress, start with toggle bit

	
	
	
		ldi command,80;
		ret;

		





fault:		ser	command		;Both "command" and "system"
		inc	system		;0xFF indicates failure
		ret;
