;***************************************************************************
;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* 
;* Title                :Sony IR Remote Control Decoder
;* Target MCU           :AT90S1200
;*
;* The timing is adapted for 4 MHz crystal
;*
;***************************************************************************




.equ	INPUT		=0	;PD0
.equ	SYS_ADDR	=0	;The system address

.equ    tbit_s		= 180	;TimerL ticks for startbitlen   
.equ    tbit		= 40	;TimerL ticks for bit*1   (34,375)
.equ    tbit2		= 80	;TimerL ticks for bit*2	  (68,75 )
.equ    tbit4		= 137	;TimerL ticks for bit*4   (137,5 )
.equ    tbit4_c		= 70	;TimerL ticks for bit*4 for test 

;	Timer tick =  16 s @ 4 MHz
;	Sony T0=550s

.equ	t2ms		= 125	;TimerL ticks for 2ms
.equ	t133ms		= 9	;TimerH ticks for 30 ms
;	TimerH tick =  4096 s @ 4 MHz



.def	S	=R5
.def	bitcounter=R26

.def	TimerH  =R28		;updated every 4096 s
.def	temp	=R31
.def	system	=R30
.def	command	=R29

.def	bitcnt	=R27




.def    error = r24



IR_Init:	;cli			;
		ldi	temp,3		;Timer/Counter 0 clocked at CK/64
		out	TCCR0,temp
		;sei			;Enable gobal iterrupt for Switch(es)
		ret 			;



;**************************************************************************
IR_REC:
		clr temp;		clear all counters and timers
		out TCNT0,temp;
		clr timerH;
		

	; *** test if idle for minimum 2ms					
IR_det_idle: 	
		in temp,TCNT0
		cpi temp,t2ms;		time done?
		brge IR_det_sb0;	yes -> jump 
	
			
		sbis PIND,Input;	LO at Input?
		rjmp fault;		Yes -> failure !
		
		rjmp IR_det_idle;	do the loop again
		

	; *** wait for startbit (maximum 30ms)


IR_det_sb0:	clr temp;		clear all counters and timers
		out TCNT0,temp;
		clr TimerH;

IR_det_sb:	in temp,TCNT0
		cpi temp,$FF;		
		brlo IR_noovf;
		inc TimerH;
		
		IR_noovf:
	
		cpi TimerH,t133ms;	30 ms without signal?	
		brlo nofault;		no  -> jump
		rjmp fault;		yes -> failure !
		
		
		nofault:
		;mov temp,sw_status;	switch pressed between?
		;cpi temp,0;
		;brne fault;		yes -> failure !
				
		sbis PIND,Input;	LO at Input?
		rjmp IR_sb;		NO -> Startbit  !		
		rjmp IR_det_sb;		Yes -> do it again
				
		
	; *** startbit starts now, should be 4x550s =To*4
	; *** Input is LO
			
IR_SB:		
		clr temp;		clear all counters and timers
		out TCNT0,temp;
		clr TimerH;
	


	; *** wait now until Startbit is short before end
IR_SB_1:	
	
		sbic PIND,Input;	Input LO?
		rjmp fault;		No -> failure

		in temp,TCNT0	
		cpi temp,tbit4_c;	Start bit short before end ?	
		brne ir_sb_1;		No -> wait again 

			
IR_SB_2:	sbic Pind,Input;	Startbit goes hi? No -> skip
		rjmp IR_readdata;	Yes -> read !
				
		in temp,TCNT0	
		cpi temp,tbit_s;	Start bit end ?	
		brne ir_sb_2;		No -> wait again 
		
			
	; *** string starts now, should be 12 bits together



		ldi bitcounter,12;	set bitcounter to 12 bits
	
IR_readdata:		
	
	
		clr temp;		clear all counters and timers
		out TCNT0,temp;

IR_waitrd_lo:			;	wait for the falling edge
		sbis Pind,Input;	if in goes lo,
		rjmp IR_wait_re;	jump to read len of bit

		in temp,TCNT0	
		cpi temp,tbit4;	To long ?	
		brne ir_waitrd_lo;	No -> wait again 
		rjmp fault;		Yes -> failure !
		
	; *** wait for rising edge
	        
IR_wait_re:	
		clr temp;		clear all counters and timers
		out TCNT0,temp;

IR_wait_re1:	in temp,TCNT0	
		cpi temp,t2ms;		Bit to long?	
		brsh fault;		Yes -> failure !
		        
	        sbis Pind,Input;
		rjmp IR_wait_re1;	wait again


		in temp,TCNT0;		read Timer

		cpi temp,tbit;		is Time
		brlo IR_bitis0;		< To -> a zero
		rjmp IR_bitis1;		> To -> a one
		
IR_bitis0:
		andi system,$7F;	clr bit 7 in system
		clc ;
		ror system;
		ror command;		
		rjmp IR_countbits;




IR_bitis1:
		ori system,128;		set bit 7 in system
		clc ;
		ror system;
		ror command;
		
IR_countbits:
		dec bitcounter;
		brne IR_readdata;
		
		clc ;
		ror system;
		ror command;
		
		clc ;
		ror system;
		ror command;
					; now cmd 1=bit 2;
		ror command;	
		ror command;
		andi command,63;	; now cmd 1=bit 0;
		
		ror system;	
		ror system;
		andi system,63;	; now cmd 1=bit 0;
		
		
		ret ;









fault:		ser	command		;Both "command" and "system"
		ser	system		;0xFF indicates failure
		ret ;
